skiff = None


def setSkiff(s):
    global skiff
    skiff = s


class SkiffSize(object):
    """SkiffSize"""
    def __init__(self, options=None, **kwargs):
        super(SkiffSize, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)

    def __repr__(self):
        return '<%s>' % (self.slug)


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/sizes', params)
    return [SkiffSize(a) for a in r['sizes']]


def get(s):
    s = str(s)
    sizes = all()
    for size in sizes:
        if s in size.slug:
            return size
    raise ValueError('Unknown Size')





class SkiffKernel(object):
    """SkiffKernel"""
    def __init__(self, options=None, **kwargs):
        super(SkiffKernel, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)

    def __repr__(self):
        return '<%s (#%s) %s>' % (self.name, self.id, self.version)




skiff = None


def setSkiff(s):
    global skiff
    skiff = s


class SkiffAction(object):
    """SkiffAction"""
    def __init__(self, options=None, **kwargs):
        super(SkiffAction, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)

    def __repr__(self):
        return '<%s (#%s) %s>' % (self.type, self.id, self.status)


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/actions', params)
    return [SkiffAction(a) for a in r['actions']]


def get(aid):
    r = skiff.get('/actions/%s' % (aid))
    if 'message' in r:
        raise ValueError(r['message'])
    else:
        return SkiffAction(r['action'])



skiff = None


def setSkiff(s):
    global skiff
    skiff = s


class SkiffRegion(object):
    """SkiffRegion"""
    def __init__(self, options=None, **kwargs):
        super(SkiffRegion, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)

    def __repr__(self):
        return '<%s (%s)>' % (self.name, self.slug)


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/regions', params)
    return [SkiffRegion(a) for a in r['regions']]


def get(r):
    regions = all()
    for region in regions:
        if r in region.slug:
            return region
    raise ValueError('No Suitable Region Found')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of skiff.
# http://github.com/Shrugs/skiff

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Matt Condon m@cond.in

# from . import (Action, Domain, Droplet, Image, Kernel, Key, Region, Size)
from .utils import rig





class SkiffNetwork(object):
    """SkiffNetwork"""
    def __init__(self, options=None, **kwargs):
        super(SkiffNetwork, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)

    def __repr__(self):
        return '<%s - %s>' % (self.ip_address, self.type)



import json
import requests

DO_BASE_URL = "https://api.digitalocean.com/v2"


class SkiffClass(object):
    """docstring for SkiffClass"""

    from . import (Action, Domain, Droplet, Image, Kernel, Key, Region, Size)

    DO_HEADERS = {
        "Content-Type": "application/json"
    }
    DO_DELETE_HEADERS = {
        "Content-Type": "application/x-www-form-urlencoded"
    }

    def __init__(self, token):
        super(SkiffClass, self).__init__()
        self.DO_TOKEN = token
        self.DO_HEADERS['Authorization'] = "Bearer %s" % (self.DO_TOKEN)
        self.DO_DELETE_HEADERS['Authorization'] = "Bearer %s" % (self.DO_TOKEN)

        self.Action.setSkiff(self)
        self.Domain.setSkiff(self)
        self.Droplet.setSkiff(self)
        self.Image.setSkiff(self)
        self.Key.setSkiff(self)
        self.Region.setSkiff(self)
        self.Size.setSkiff(self)

    def get(self, url, params=None):
        if not params:
            params = {}
        r = requests.get(DO_BASE_URL + url, params=params, headers=self.DO_HEADERS)
        return r.json()

    def post(self, url, data=None):
        if not data:
            data = {}

        r = requests.post(DO_BASE_URL + url, data=json.dumps(data), headers=self.DO_HEADERS)
        return r.json()

    def delete(self, url):
        r = requests.delete(DO_BASE_URL + url, headers=self.DO_DELETE_HEADERS)
        return r.status_code == 204

    def put(self, url, data=None):
        if not data:
            data = {}

        r = requests.put(DO_BASE_URL + url, data=json.dumps(data), headers=self.DO_HEADERS)
        return r.json()


def rig(token):
    return SkiffClass(token)



skiff = None


def setSkiff(s):
    global skiff
    skiff = s


class SkiffImage(object):
    """SkiffImage"""
    def __init__(self, options=None, **kwargs):
        super(SkiffImage, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)

    def __repr__(self):
        return '<%s (#%s) %s>' % (self.name, self.id, self.distribution)

    def do_action(self, action, options=None):
        if not options:
            options = {}

        if isinstance(action, SkiffAction):
            action = action.type

        options['type'] = action

        r = skiff.post('/images/%s/actions' % (self.id), data=options)
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffAction(r['action'])

    def transfer(self, region):
        if isinstance(region, SkiffRegion):
            region = region.slug

        return self.do_action('transfer', {'region': region})

    def delete(self):
        return skiff.delete('/images/%s' % (self.id))

    def update(self, new_name):
        options = {
            'name': new_name
        }

        r = skiff.put('/images/%s' % (self.id), data=options)
        self.name = new_name
        return SkiffImage(r['image'])

    def actions(self):
        r = skiff.get('/images/%s/actions')
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return [SkiffAction(a) for a in r['actions']]

    def get_action(self, action_id):
        if isinstance(action_id, SkiffAction):
            action_id = action.id

        r = skiff.get('/images/%s/actions/%s' % (self.id, str(action)))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffAction(r['action'])


def get(iid):
    # same endpoint works with ids and slugs
    r = skiff.get('/images/%s' % (iid))
    if 'message' in r:
        # could not find, try basic search
        images = all()
        for img in images:
            if iid in img.name:
                return img
        raise ValueError(r['message'])
    else:
        return SkiffImage(r['image'])


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/images', params)
    if 'message' in r:
        raise ValueError(r['message'])
    else:
        return [SkiffImage(d) for d in r['images']]



from .Action import SkiffAction
from .Image import SkiffImage
from .Size import SkiffSize
from .Kernel import SkiffKernel
from .Region import SkiffRegion
from .Network import SkiffNetwork
from .Key import SkiffKey
from . import Domain

skiff = None


def setSkiff(s):
    global skiff
    skiff = s


def destroy_droplet(did):
    return skiff.delete('/droplets/' + str(did))


class SkiffDroplet(object):
    """SkiffDroplet"""
    def __init__(self, options):
        super(SkiffDroplet, self).__init__()
        # create instance methods for everything in options
        self.__dict__.update(options)
        # droplet snapshots, backups, actions, networks
        # @TODO: make this cleaner?
        self.region = SkiffRegion(options['region'])
        self.image = SkiffImage(options['image'])
        self.size = SkiffSize(options['size'])
        self.kernel = SkiffKernel(options['kernel'])
        for network_type, networks in options['networks'].iteritems():
            setattr(self, network_type, [SkiffNetwork(n) for n in networks])

        # aliases
        self.restart = self.reboot
        self.action = self.get_action
        self.reset_password = self.password_reset
        self.refresh = self.reload

    def __repr__(self):
        return '<%s (#%d) %s - %s - %s' % (self.name, self.id, self.region.slug, self.image.name, self.size.slug)

    def do_action(self, action, options=None):
        if not options:
            options = {}

        if isinstance(action, SkiffAction):
            action = action.type

        options['type'] = action

        r = skiff.post('/droplets/%s/actions' % (self.id), data=options)
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffAction(r['action'])

    def destroy(self):
        return destroy_droplet(self.id)

    def kernels(self):
        r = skiff.get('/droplets/%s/kernels' % (self.id))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return [SkiffKernel(a) for a in r['kernels']]

    def snapshots(self):
        r = skiff.get('/droplets/%s/snapshots' % (self.id))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return [SkiffImage(a) for a in r['snapshots']]

    def backups(self):
        r = skiff.get('/droplets/%s/backups' % (self.id))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return [SkiffImage(a) for a in r['backups']]

    def actions(self):
        r = skiff.get('/droplets/%s/actions' % (self.id))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return [SkiffAction(a) for a in r['actions']]

    def reboot(self):
        return self.do_action('reboot')

    def power_cycle(self):
        return self.do_action('power_cycle')

    def shutdown(self):
        return self.do_action('shutdown')

    def power_off(self):
        return self.do_action('power_off')

    def power_on(self):
        return self.do_action('power_on')

    def password_reset(self):
        return self.do_action('password_reset')

    def resize(self, slug):
        # if is of class SkiffSize, grab its property
        if isinstance(slug, SkiffSize):
            slug = slug.slug
        return self.do_action('resize', {'size': slug})

    def restore(self, backup):
        # if is of class SkiffImage, grab its property
        if isinstance(backup, SkiffImage):
            backup = backup.id
        return self.do_action('resize', {'image': backup})

    def rebuild(self, image=None):
        # if is of class SkiffImage, grab its property
        if image and isinstance(image, SkiffImage):
            image = image.id
        elif not image:
            image = self.image.id
        return self.do_action('rebuild', {'image': image})

    def rename(self, new_name):
        r = self.do_action('rename', {'name': new_name})
        self.name = new_name
        return r

    def change_kernel(self, kernel):
        # if kernel is of class SkiffKernel, grab its property
        if isinstance(kernel, SkiffKernel):
            kernel = kernel.id
        return self.do_action('change_kernel', {'kernel': kernel})

    def enable_ipv6(self):
        return self.do_action('enable_ipv6')

    def disable_backups(self):
        return self.do_action('disable_backups')

    def enable_private_networking(self):
        return self.do_action('enable_private_networking')

    def get_action(self, action_id):
        r = skiff.get('/droplets/%s/actions/%s' % (self.id, action_id))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffAction(r['action'])

    # UTILITY METHODS
    def has_action_in_progress(self):
        for action in self.actions():
            if action.status == 'in-progress':
                return True

        return False

    def reload(self):
        return get(self.id)

    def wait_till_done(self, t=5):
        import time
        while self.has_action_in_progress():
            time.sleep(t)

    def create_domain(self, domain_name):
        return Domain.create(name=domain_name, ip_address=self.v4[0].ip_address)


def get(did):
    if type(did).__name__ == 'int':
        # is droplet id, get it
        r = skiff.get('/droplets/%s' % (did))
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffDroplet(r['droplet'])
    else:
        # is droplet name, search
        # @TODO: use fuzzy search or something more intelligent
        ds = all()
        for d in ds:
            if did in d.name:
                return d

        raise ValueError('No Suitable Droplet Found for Search: %s' % did)


def create(options=None, **kwargs):
    if not options:
        options = kwargs

    for key, val in options.iteritems():
        if key == 'region' and isinstance(val, SkiffRegion):
            options['region'] = val.slug
        elif key == 'size' and isinstance(val, SkiffSize):
            options['size'] = val.slug
        elif key == 'image' and isinstance(val, SkiffImage):
            options['image'] = val.id
        elif key == 'ssh_keys':
            for i, ssh_key in enumerate(val):
                if isinstance(ssh_key, SkiffKey):
                    options['ssh_keys'][i] = ssh_key.id

    r = skiff.post('/droplets', data=options)
    if 'message' in r:
        raise ValueError(r['message'])
    else:
        return SkiffDroplet(r['droplet'])


# alias new to create
new = create


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/droplets', params)
    if 'message' in r:
        # @TODO: Better error?
        raise ValueError(r['message'])
    else:
        # create new Droplets for each droplet
        return [SkiffDroplet(d) for d in r['droplets']]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of skiff.
# http://github.com/Shrugs/skiff

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Matt Condon m@cond.in


from setuptools import setup, find_packages
import os
import sys

# Publish Helper.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
]

setup(
    name='skiff',
    version='0.9.1',
    description="Python library for DigitalOcean's v2 API",
    long_description='''
Python library for DigitalOcean's v2 API
''',
    keywords='digitalocean api library v2',
    author='Matt Condon',
    author_email='m@cond.in',
    url='http://github.com/Shrugs/skiff',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: Implementation :: PyPy",
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        # add your dependencies here
        # remember to use 'package-name>=x.y.z,<x.y+1.0' notation (this way you get bugfixes)
        'requests>=2.3.0,<3.0.0'
    ],
    extras_require={
        'tests': tests_require
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            # 'skiff=skiff.cli:main',
        ],
    },
)



skiff = None


def setSkiff(s):
    global skiff
    skiff = s


class SkiffKey(object):
    """SkiffKey"""
    def __init__(self, options=None, **kwargs):
        super(SkiffKey, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)
        self.delete = self.destroy

    def __repr__(self):
        return '<%s (#%s)>' % (self.name, self.id or '??')

    def update(self, new_name):
        options = {
            'name': new_name
        }

        r = skiff.put('/account/keys/%s' % (self.id), data=options)
        self.name = new_name
        return SkiffKey(r['ssh_key'])

    def destroy(self):
        return skiff.delete('/account/keys/%s' % (self.id))


def get(kid):
    r = skiff.get('/account/keys/%s' % (kid))
    if 'message' in r:
        # try searching
        keys = all()
        for key in keys:
            if kid in key.name:
                return key
        raise ValueError(r['message'])
    else:
        return SkiffKey(r['ssh_key'])


def create(options=None, **kwargs):
    if not options:
        options = kwargs

    r = skiff.post('/account/keys', data=options)
    if 'message' in r:
        raise ValueError(r['message'])
    else:
        return SkiffKey(r['ssh_key'])


# alias new to create
new = create


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/account/keys', params)
    if 'message' in r:
        # @TODO: Better error?
        raise ValueError(r['message'])
    else:
        # create new account/keys for each droplet
        return [SkiffKey(d) for d in r['ssh_keys']]



skiff = None


def setSkiff(s):
    global skiff
    skiff = s


class SkiffDomainRecord(object):
    """SkiffDomainRecord"""
    def __init__(self, domain, options=None, **kwargs):
        super(SkiffDomainRecord, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)
        self.domain = domain
        self.destroy = self.delete

    def __repr__(self):
        return '<%s - %s (#%s) %s -> %s>' % (self.domain.name, self.type, self.id, self.name, self.data)

    def delete(self):
        return self.domain.delete_record(self.id)

    def update(self, new_name):
        self.name = new_name
        return self.domain.update_record(self.id, new_name)


def delete_domain(did):
    return skiff.delete('/domains/%s' % (did))


class SkiffDomain(object):
    """SkiffDomain"""
    def __init__(self, options=None, **kwargs):
        super(SkiffDomain, self).__init__()
        if not options:
            options = kwargs

        self.__dict__.update(options)
        self.record = self.get_record
        self.create = self.create_record
        self.destroy = self.delete

    def __repr__(self):
        return '<%s>' % (self.name)

    def delete(self):
        return delete_domain(self.name)

    def records(self):
        r = skiff.get('/domains/%s/records' % (self.name))

        return [SkiffDomainRecord(self, record) for record in r['domain_records']]

    def create_record(self, options=None, **kwargs):
        if not options:
            options = kwargs

        r = skiff.post('/domains/%s/records' % (self.name), data=options)
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffDomainRecord(self, r['domain_record'])

    def delete_record(self, domain, record):
        # if is SkiffDomainRecord, grab that property
        if isinstance(record, SkiffDomainRecord):
            record = record.id

        return skiff.delete('/domains/%s/records/%s' % (self.name, record))

    def get_record(self, record):
        r = skiff.get('/domains/%s/records/%s' % (self.name, record))
        return SkiffDomainRecord(self, r['domain_record'])

    def update_record(self, record, new_name):
        if isinstance(record, SkiffDomainRecord):
            record = record.id

        options = {
            'name': new_name
        }

        r = skiff.put('/domains/%s/records/%s' % (self.name, record), data=options)
        if 'message' in r:
            raise ValueError(r['message'])
        else:
            return SkiffDomainRecord(self, r['domain_record'])


def all(params=None, **kwargs):
    if not params:
        params = kwargs

    r = skiff.get('/domains', params)
    return [SkiffDomain(a) for a in r['domains']]


def create(options=None, **kwargs):
    if not options:
        options = kwargs

    r = skiff.post('/domains', data=options)
    if 'message' in r:
        raise ValueError(r['message'])
    else:
        return SkiffDomain(r['domain'])


def get(d):
    if type(d).__name__ == 'int':
        r = skiff.get('/domains/%s' % (d))
        return SkiffDomain(r['domain'])
    else:
        # search in string
        domains = all()
        for dom in domains:
            if d in dom.name:
                return dom
        raise ValueError('No Suitable Domain Found')
